var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/* eslint-disable no-param-reassign, import/prefer-default-export */

import { GraphQLID, GraphQLNonNull } from 'graphql';
import { getProjectionFromAST } from 'graphql-compose';
import { fromGlobalId } from './globalId';
import NodeInterface from './nodeInterface';


// this fieldConfig must be set to RootQuery.node field
export function getNodeFieldConfig(typeToFindByIdMap) {
  return {
    description: 'Fetches an object that has globally unique ID among all types',
    type: NodeInterface,
    args: {
      id: {
        type: new GraphQLNonNull(GraphQLID),
        description: 'The globally unique ID among all types'
      }
    },
    resolve: function resolve(source, args, context, info) {
      if (!args.id || !(typeof args.id === 'string')) {
        return null;
      }

      var _fromGlobalId = fromGlobalId(args.id);

      var type = _fromGlobalId.type;
      var id = _fromGlobalId.id;


      var findById = typeToFindByIdMap[type];
      if (findById && findById.resolve) {
        var _ret = function () {
          var tc = findById.getTypeComposer();
          var graphqlType = tc.getType();

          // set `returnType` to `info` for proper work of `getProjectionFromAST`
          // it will correctly add required fields for `relation` to `projection`
          var projection = void 0;
          if (info) {
            info.returnType = graphqlType;
            projection = getProjectionFromAST(info);
          } else {
            projection = {};
          }

          // suppose that first argument is argument with id field
          var idArgName = Object.keys(findById.args)[0];
          return {
            v: findById.resolve({
              source: source,
              args: _defineProperty({}, idArgName, id), // eg. mongoose has _id fieldname, so should map
              context: context,
              info: info,
              projection: projection
            }).then(function (res) {
              if (!res) return res;
              res.__nodeType = graphqlType;
              return res;
            })
          };
        }();

        if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
      }

      return null;
    }
  };
}