'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _defineProperty2 = require('babel-runtime/core-js/object/define-property');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _iterator = require('babel-runtime/core-js/symbol/iterator');

var _iterator2 = _interopRequireDefault(_iterator);

var _symbol = require('babel-runtime/core-js/symbol');

var _symbol2 = _interopRequireDefault(_symbol);

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof _symbol2.default === "function" && typeof _iterator2.default === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof _symbol2.default === "function" && obj.constructor === _symbol2.default ? "symbol" : typeof obj; };

exports.getNodeFieldConfig = getNodeFieldConfig;

var _graphql = require('graphql');

var _graphqlCompose = require('graphql-compose');

var _globalId = require('./globalId');

var _nodeInterface = require('./nodeInterface');

var _nodeInterface2 = _interopRequireDefault(_nodeInterface);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { (0, _defineProperty3.default)(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
/* eslint-disable no-param-reassign, import/prefer-default-export */

// this fieldConfig must be set to RootQuery.node field
function getNodeFieldConfig(typeToFindByIdMap) {
  return {
    description: 'Fetches an object that has globally unique ID among all types',
    type: _nodeInterface2.default,
    args: {
      id: {
        type: new _graphql.GraphQLNonNull(_graphql.GraphQLID),
        description: 'The globally unique ID among all types'
      }
    },
    resolve: function resolve(source, args, context, info) {
      if (!args.id || !(typeof args.id === 'string')) {
        return null;
      }

      var _fromGlobalId = (0, _globalId.fromGlobalId)(args.id);

      var type = _fromGlobalId.type;
      var id = _fromGlobalId.id;


      var findById = typeToFindByIdMap[type];
      if (findById && findById.resolve) {
        var _ret = function () {
          var tc = findById.getTypeComposer();
          var graphqlType = tc.getType();

          // set `returnType` to `info` for proper work of `getProjectionFromAST`
          // it will correctly add required fields for `relation` to `projection`
          var projection = void 0;
          if (info) {
            info.returnType = graphqlType;
            projection = (0, _graphqlCompose.getProjectionFromAST)(info);
          } else {
            projection = {};
          }

          // suppose that first argument is argument with id field
          var idArgName = (0, _keys2.default)(findById.args)[0];
          return {
            v: findById.resolve({
              source: source,
              args: _defineProperty({}, idArgName, id), // eg. mongoose has _id fieldname, so should map
              context: context,
              info: info,
              projection: projection
            }).then(function (res) {
              if (!res) return res;
              res.__nodeType = graphqlType;
              return res;
            })
          };
        }();

        if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
      }

      return null;
    }
  };
}